"""Stub file for the 'strop' module."""

from typing import List, Sequence

lowercase: str
uppercase: str
whitespace: str

def atof(a: str) -> float:
    raise DeprecationWarning()

def atoi(a: str, base: int = ...) -> int:
    raise DeprecationWarning()

def atol(a: str, base: int = ...) -> long:
    raise DeprecationWarning()

def capitalize(s: str) -> str:
    raise DeprecationWarning()

def count(s: str, sub: str, start: int = ..., end: int = ...) -> int:
    raise DeprecationWarning()

def expandtabs(string: str, tabsize: int = ...) -> str:
    raise DeprecationWarning()
    raise OverflowError()

def find(s: str, sub: str, start: int = ..., end: int = ...) -> int:
    raise DeprecationWarning()

def join(list: Sequence[str], sep: str = ...) -> str:
    raise DeprecationWarning()
    raise OverflowError()

def joinfields(list: Sequence[str], sep: str = ...) -> str:
    raise DeprecationWarning()
    raise OverflowError()

def lower(s: str) -> str:
    raise DeprecationWarning()

def lstrip(s: str) -> str:
    raise DeprecationWarning()

def maketrans(frm: str, to: str) -> str: ...

def replace(s: str, old: str, new: str, maxsplit: int = ...) -> str:
    raise DeprecationWarning()

def rfind(s: str, sub: str, start: int = ..., end: int = ...) -> int:
    raise DeprecationWarning()

def rstrip(s: str) -> str:
    raise DeprecationWarning()

def split(s: str, sep: str, maxsplit: int = ...) -> List[str]:
    raise DeprecationWarning()

def splitfields(s: str, sep: str, maxsplit: int = ...) -> List[str]:
    raise DeprecationWarning()

def strip(s: str) -> str:
    raise DeprecationWarning()

def swapcase(s: str) -> str:
    raise DeprecationWarning()

def translate(s: str, table: str, deletechars: str = ...) -> str:
    raise DeprecationWarning()

def upper(s: str) -> str:
    raise DeprecationWarning()
